@@ -1,4 +1,5 @@
 """Environment for the xonsh shell."""
+
 import collections.abc as cabc
 import contextlib
 import inspect
@@ -407,9 +408,11 @@ class LsColors(cabc.MutableMapping):
                     + "="
                     + ";".join(
                         [
-                            LsColors.target_value
-                            if key in self._targets
-                            else ansi_color_name_to_escape_code(v, cmap=style)
+                            (
+                                LsColors.target_value
+                                if key in self._targets
+                                else ansi_color_name_to_escape_code(v, cmap=style)
+                            )
                             for v in val
                         ]
                     )
@@ -482,7 +485,7 @@ class LsColors(cabc.MutableMapping):
             out = subprocess.check_output(
                 cmd, env=denv, text=True, stderr=subprocess.DEVNULL
             )
-        except (subprocess.CalledProcessError, FileNotFoundError):
+        except (subprocess.CalledProcessError, FileNotFoundError, NotADirectoryError):
             return cls(cls.default_settings)
         except OSError:
             # necessary to catch OSError: [WinError 740] The requested operation requires elevation
@@ -862,12 +865,12 @@ class GeneralSetting(Xettings):
 
     ENABLE_COMMANDS_CACHE = Var(
         default=True,
-        doc="command names in a directory are cached when enabled.",
-        doc_default="True"
+        doc="command names in a directory are cached when enabled. "
         "On some platforms it may not be accurate enough"
         "(e.g. Windows, Linux save mtime in seconds). "
-        "setting it to False would disable the caching mechanism "
+        "Setting it to False would disable the caching mechanism "
         "and may slow down the shell",
+        doc_default="True",
     )
 
     HOSTNAME = Var.with_default(
@@ -1689,7 +1692,14 @@ class PTKSetting(PromptSetting):  # sub-classing -> sub-group
     )
     XONSH_COPY_ON_DELETE = Var.with_default(
         False,
-        "Whether to copy words/lines to clipboard on deletion (must be set in .xonshrc file)."
+        "Whether to copy words/lines to clipboard on deletion (must be set in the run control file)."
+        "Does not have any effect in ``vi_mode``."
+        "Only available under the prompt-toolkit shell.",
+    )
+    XONSH_USE_SYSTEM_CLIPBOARD = Var.with_default(
+        True,
+        "Whether to let the shell use the system clipboard (must be set in the run control file)."
+        "The main use-case is to fully disable clipboard integration in ``vi_mode``."
         "Only available under the prompt-toolkit shell.",
     )
     XONSH_CTRL_BKSP_DELETION = Var.with_default(
@@ -1802,6 +1812,15 @@ This is to reduce the noise in generated completions.""",
         "Toggles subsequence matching of paths for tab completion. "
         "If ``True``, then, e.g., ``~/u/ro`` can match ``~/lou/carcolh``.",
     )
+    XONSH_BASH_PATH_OVERRIDE = Var.with_default(
+        None,
+        doc="Override the path used to call bash externally. This allows "
+        "a user to choose which bash executable to call for ``source-bash`` "
+        "and any bash completions that are configured. "
+        "This variable overrides the default settings in "
+        "``xonsh.platform.bash_command``.",
+        doc_default="None",
+    )
 
 
 class PTKCompletionSetting(AutoCompletionSetting):
